#!/usr/bin/python3
import cv2, os 
from cv2 import face
import numpy as np
from comfuncs import common                    # own import

( verbose, threshold, dataset, source ) = common.setParams()
if verbose:
    print( 'verbose={} , threshold={} , dataset={} , source={}'.\
           format( verbose, threshold, dataset, source ) )

cap = common.getCapture()                             # create VideoCapture object for camera or file
if verbose and cv2.__version__ >= '3.3':
    if not cv2.VideoCapture.isOpened( cap ):
        print( "no source captured!" )
        sys.exit( 1 )
    else:
        print( "capture open OK!" )

NewName = raw_input( 'Enter Name: ' ).strip()
#NewName = input( 'Enter Name: ' ).strip()
csv = dataset + '.csv'                                        # read from CSV track of ID's and Names.
base = 0                                                             # frame number, 0 -> new user name
if not os.path.exists( csv ):                               # create a new database
    if not os.path.isdir( dataset ):
        os.mkdir( dataset )
    else:
        print( 'dublicated dataset name' )
        sys.exit( 1 )
    file = open( csv, "wt" )
    CurrentId   = 1
    CurrentName = NewName
else:
    try:                                                                  # read the existing database
        file = open( csv, "rt" )
        data = file.readlines()
    except IOError as err:
        print( err )
        sys.exit( 1 )
    CurrentId   = 0
    CurrentName = ''
    for line in data:
        if data[ 0 ] == ord( '#' ): continue          # comment line
        words = line.split( ',' )
        if 2 == len( words ):
            CurrentId = int( words[ 0 ] )
            CurrentName = words[ 1 ].strip()
            if NewName == CurrentName:
                _, Ids = common.getImagesAndLabels( dataset )
                if verbose: print( Ids )
                i1 = filter( lambda x: x == CurrentId, Ids )
                if verbose: print( i1 )
                base = len( i1 ) + 1
                break
    if NewName != CurrentName:                       # NewName not found in database
        CurrentId += 1
        CurrentName = NewName
file.close
#print( base )
print( '-------------------------------------------------------------' )
print( 'Press Spase - for save, Esc or "q" - for quit' )

max_count = 10                                                  # maximum count for the user
count = 0

if not os.path.isdir( dataset ):
    os.mkdir( dataset )

while count <= max_count:                              # break if count value exceeds set
    ret, frame = cap.read()
    if not ret: break
    if type( frame ) != np.ndarray:
        print( 'canceled !' )
        sys.exit( 0 )
    if frame.ndim > 2 :                                         # convert BGR to grayscale (if not yet)
        gray = cv2.cvtColor( frame, cv2.COLOR_BGR2GRAY )
    faces = common.detect( frame )
    for x, y, w, h in faces:                                    # draw rectangle around the face
        cv2.rectangle( frame, ( x, y ), ( x + w, y + h ), ( 255, 0, 0 ), 2 )
    cv2.imshow( '', frame )                                   # show the captured frame
    key = cv2.waitKey( 200 ) & 0xFF
    if key == ord( ' ' ): 
        if 1 == len( faces ):                                   # save the captured face in dataset folder
            count += 1                                              # increment frames count
            path = '{}{}{:03d}.{:02d}.jpg'.format( dataset, os.sep, CurrentId, base + count ) 
            cv2.imwrite( path, gray[ y : y + h, x : x + w ] )
            print( 'captured frame {}'.format( count ) )
    elif( ( 27 == key ) or ( key == ord( 'q' ) ) ): # Esc or 'q'
        break
cv2.destroyAllWindows()                                     # do cleanup
cap.release()  

if 0 == base and count > 1:                               # only if frames were captured update CVS
    file = open( csv, "at" )
    file.write( '{:03d} , {}\n'.format( CurrentId, CurrentName.strip() ) )
    file.close
if verbose:
    print( '{}: {} - added {} images'.format( CurrentId, CurrentName, count ) )

print "Wait for the program to finish training..."
time = float( cv2.getTickCount() )
faces, Ids = common.getImagesAndLabels( dataset )     # get faces and Ids from the dataset folder
if cv2.__version__ < '3.4':                                    # create LBPH recognizer
    recognizer = face.createLBPHFaceRecognizer( 1, 8, 8, 8, threshold )
else:
    recognizer = face.LBPHFaceRecognizer_create( 1, 8, 8, 8, threshold )
recognizer.train( faces, np.array( Ids ) )  
time = ( float( cv2.getTickCount() ) - time ) / cv2.getTickFrequency() 
recognizer.save( dataset + '.yml' )                      # save the recognizer out in .yml
if verbose:
    print( 'traning time was {} seconds'.format( time ) )
