#!/usr/bin/python3
import cv2, os, sys
from cv2 import face
from comfuncs import common                     # own import
from actions import action_yes, action_no, action_free, trouble

( verbose, threshold, dataset, source ) = common.setParams()
if verbose:
    print( 'verbose={} , threshold={} , dataset={} , source={}'
           .format( verbose, threshold, dataset, source ) )

csvfile = dataset + '.csv'
if not os.path.isfile( csvfile ):
    print( 'CSV file {} not found'.format( csvfile ) )
    sys.exit( 1 )

cap = common.getCapture()                             # create VideoCapture object for camera or file

if cv2.__version__ < '3.4':
    recognizer = face.createLBPHFaceRecognizer( 1, 8, 8, 8, threshold )
else:
    recognizer = face.LBPHFaceRecognizer_create( 1, 8, 8, 8, threshold )

file = open( csvfile, "r" )                                  # read the file with Id & Names - CSV
data = file.readlines()
UserDict = {}
IdentDict = {}
for line in data:
    words = line.split( "," )
    if 2 == len( words ):
        CurrentId = words[ 0 ].strip()
        CurrentName = words[ 1 ].strip()
        if verbose:
            print( CurrentId, CurrentName ) 
        UserDict[ int( CurrentId ) ] = CurrentName
        IdentDict[ int( CurrentId ) ] = 0
file.close
if verbose:
    print( UserDict )
    print( '-------------------------------------------------------------' )
    print( 'Load the recognizer' )

try:
    if cv2.__version__ < '3.4':
        recognizer.load( dataset + '.yml' )
    else:
        recognizer.read( dataset + '.yml' )
except:
    print( 'you must re-train the classifier {}!'.format( dataset + '.yml' ) )
    sys.exit( 1 )

fontFace = cv2.FONT_HERSHEY_SIMPLEX
meant = 0.0
dispt = 0.0
numAll = 0 ; numOk = 0 ; numAns = 0 ; NumNo = 0
while True:
    ret, frame = cap.read()                                  # capture the frame
    if not ret: break
    if frame.ndim > 2:                                         # convert from BGR to grayscale
        gray = cv2.cvtColor( frame, cv2.COLOR_BGR2GRAY )
    else:
        gray = frame        
    time = float( cv2.getTickCount() )
    faces = common.detect( frame )                   # detect faces of different sizes
    if 1 == len( faces ):                                       # only single face
        numAll += 1
        for ( x, y, w, h ) in faces:                            # draw rectangle at the face
            cv2.rectangle( frame, ( x, y ), ( x + w, y + h ), ( 225, 0, 0 ), 2 )
            try:                                                          # identify the face
                Id, conf = recognizer.predict( gray[ y : y + h, x : x + w ] )
            except:
                print( 'you must re-train the classifier {}!'.format( dataset + '.yml' ) )
                sys.exit( 1 )
#           print( Id, conf )
            if Id < 0: 
                user = 'Unknown' 
                NumNo += 1
                fontColor = ( 0, 0, 255 )       # RED
            elif conf < threshold:
                user = UserDict[ Id ]
                IdentDict[ Id ] += 1                
                numOk += 1
                fontColor = ( 0, 255, 0 )       # GREEN
            else:
                user = 'Aambiguous' 
                numAns += 1
                fontColor = ( 225, 0, 0 )       # BLUE
            time = ( cv2.getTickCount() - time ) / cv2.getTickFrequency() 
            meant += time
            dispt += time * time
            cv2.rectangle( frame, ( x, y ), ( x + w, y + h ), fontColor , 2 )
            cv2.putText( frame, user, ( x, y + h ), fontFace, 1, fontColor, 2 )
            if verbose:
                print( "{:3} {:20} : {}".format( Id, conf, user ) )
            if 'Unknown' == user or 'Aambiguous' == user:
                trouble( frame )
    cv2.imshow( '', frame )                     # show the captured frame in all cases
    key = cv2.waitKey( 100 ) & 0xFF 
    if 27 == key or key == ord( 'q' ): break 
cap.release()
cv2.destroyAllWindows()

print( 'detected {} persons of them {} identified, {} not identified, {} in doubt'.
       format( numAll, numOk, NumNo, numAns ) )
for i in IdentDict.keys():
    if IdentDict[ i ] != 0:
        print( 'identified as {}: {} times'.format( UserDict[ i ], IdentDict[ i ] ) )
print( 'the average identification time was {} seconds'.format( meant / numAll ) )
